/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             Virtual table Management
*  SCOPE :              Implements Virtual table management
*  FILE :               virtualtable.hxx
*  AUTHORS :            E. Abecassis & L. Lesaint
*  CREATION DATE :      August 24, 1994
*  COPYRIGHTS :        (C) APIC SYSTEMES, 1994
*  HISTORY :
*
******************************************************************************/

/* IMPORTS *******************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

#ifndef __virtualtable_HXX
#define __virtualtable_HXX

#define yoo_alignSup(ind,step) (((ind / step)+ 1) * step)

template <class ELEM> class VirtualTable {

  static ELEM* _stock;
  static unsigned int _stock_size;
  
  unsigned int _step ;
  unsigned int _size ;
  ELEM* _tab ;
  
  public :

  static void InitClass() {
    VirtualTable<ELEM>::_stock=NULL;
    VirtualTable<ELEM>::_stock_size=0;
  }
  VirtualTable(int s) : _size(s), _step(s) {
    if(!VirtualTable<ELEM>::_stock) {
      _tab = new ELEM[this->_step] ;
    }
    else {
      _tab= VirtualTable<ELEM>::_stock;
      _size= VirtualTable<ELEM>::_stock_size;
      VirtualTable<ELEM>::_stock= NULL;
    }
  }
  VirtualTable(const VirtualTable<ELEM>& vt): _tab(NULL) { *this = vt ; }
  ~VirtualTable() {
    if(!VirtualTable<ELEM>::_stock) {
      VirtualTable<ELEM>::_stock= this->_tab;
      VirtualTable<ELEM>::_stock_size= this->_size;
      this->_tab= NULL;
    }
    else {
      if(this->_tab) delete this->_tab ;
    }
  }
  
  VirtualTable& operator=(const VirtualTable<ELEM>& vt) {
    if(this->_tab) delete this->_tab ;
    this->_tab = new ELEM[vt._size] ;
    this->_size = vt._size ;
    this->_step = vt._step ;
    memcpy(this->_tab, vt._tab, vt._size*sizeof(ELEM)) ;
    return *this ;
  }
       
  inline ELEM& operator[](int ind) {
    if (ind < this->_size) {
      return _tab[ind] ;
    }
    int oldsize = this->_size ;
    this->_size = yoo_alignSup(ind,this->_step) ;
    ELEM* nouv = new ELEM[this->_size] ;
    memcpy(nouv, this->_tab, oldsize*sizeof(ELEM)) ;
    if(this->_tab) delete this->_tab ;
    this->_tab = nouv ;
    return _tab[ind] ;
  }

  operator ELEM*() { return _tab ; } 
} ;

#endif
